/*
Copyright (C) 2000 David G. Watson

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
	 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
				  
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "mainwin.h"
#include <qfiledialog.h>
#include <qstring.h>
#include <iostream>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qlabel.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <qfile.h>
#include <qsocket.h>
#include <qmessagebox.h>
#include <fstream>
#include <qprogressdialog.h>
#include <signal.h>


void MainWin::otherSetup() {
	curDir = "/";
	TotalSizeLabel->setText("0 bytes");

//	setAcceptDrops(true);
}

void MainWin::calcSize() {
	totalSizeBytes = 0;

	for (int i = 0; i < ListBox1->count(); i++) {
		QString s = ListBox1->text(i);
//		clog << "calculating size of: " << s << endl;

		QDir d(s, "*.mp3 *.MP3 *.Mp3 *.mP3");
		QStringList sl = d.entryList();
		
		int mySize = 0;

		for (QStringList::Iterator it = sl.begin(); it != sl.end(); it++) {
			mySize += QFileInfo(s + *it).size();
		}
		
		totalSizeBytes += mySize;	
	}
	TotalSizeLabel->setText(QString("%1 MB").arg(totalSizeBytes/1048576));
}

void MainWin::SlotAdd()
{

      QString dir = QFileDialog::getExistingDirectory(curDir, this, "getDirDlg", "Choose a Directory", false); 

      if (!dir.isNull()) {
	    ListBox1->insertItem(new QListBoxText(dir));

	    RemoveButton->setEnabled(true);
	    curDir = dir;

	    calcSize();
      }
}

void MainWin::SlotCreateImage()
{
	if (!CreateImageCheckErrors()) {
	
		int socket1[2], socket2[2];

		if (pipe(socket1) || pipe(socket2)) {
			qWarning("Something terrible happened in MainWin::SlotCreateImage!");
			exit(1);
		}

//		clog << "MainWin::SlotCreateImage: pipe successful\n";	

		pid = fork();

		switch(pid) {
			case -1: // Error
				qWarning("Could not fork in MainWin::SlotCreateImage");
				exit(1);
				break;
			case 0: // Child
				readPipe = socket1[0];
				writePipe = socket2[1];
//				close(socket2[0]);
//				close(socket1[1]);
//				clog << "MainWin::SlotCreateImage: fork successful on child\n";
				DoChildStuff();

				exit(0);
				break;
			default: // Parent
				readPipe = socket2[0];
				writePipe = socket1[1];
//				close(socket1[0]);
//				close(socket2[1]);
//				clog << "MainWin::SlotCreateImage: fork successful on parent\n";
				DoParentStuff();
		}
	}
}


void MainWin::SlotMoveDown()
{
    int index = ListBox1->currentItem();

    if (index != -1) {
	QString text = ListBox1->currentText();

	ListBox1->removeItem(index);
	ListBox1->insertItem(text, index + 1);
	ListBox1->setCurrentItem(index + 1);
    }
}

void MainWin::SlotMoveUp()
{
    int index = ListBox1->currentItem();

    if (index > 0) {
	QString text = ListBox1->currentText();

	ListBox1->removeItem(index);
	ListBox1->insertItem(text, index - 1);
	ListBox1->setCurrentItem(index - 1);
    }

}

void MainWin::SlotRemove()
{
    int index = ListBox1->currentItem();
    
    if (index != -1) {
	    ListBox1->removeItem(index);
	    calcSize();
    }
}

void MainWin::SlotSelectImageLocation()
{
      QString f = QFileDialog::getSaveFileName(); 

      if (!f.isNull()) {
	    ImageLocationEntry->setText(f);
      }
	
}


		
void MainWin::DoChildStuff()
{
	dup2(readPipe, 0);
	dup2(writePipe, 1);
	dup2(writePipe, 2);

	// Here, we're making something of the form "/path/to/file/on/cd=/path/to/file/on/disk"
	QDir d(ListBox1->text(0), "*.mp3 *.MP3 *.Mp3 *.mP3");
	QString temp = ListBox1->text(0);
	temp.truncate(temp.length()-1);
	temp = temp.right(temp.length() - temp.findRev('/') -1);
	QString s = QString("01") + temp + '/' + d.entryList().first() + '=' + ListBox1->text(0) 
		+ d.entryList().first();
	
	char *str = new char[s.length() +1];
	strcpy(str, s.latin1());

	bool flag = true; // We don't want to print out the very first entry, since it's give on the command line
	
	char *tmp = tempnam("/tmp/", "cdcreate-");
	ofstream out(tmp);

	for (int i = 0; i < ListBox1->count(); i++) {
		QString s;

		if (i < 9) {
			s = ListBox1->text(i);
			s.truncate(s.length()-1);
			s = s.right(s.length() - s.findRev('/') - 1); 

			s = QString("0%1").arg(i+1) + s;
		} else {
			s = ListBox1->text(i);
			s.truncate(s.length()-1);
			s = s.right(s.length() - s.findRev('/') - 1); 

			s = QString("%1").arg(i+1) + s;
		}

		QString t = ListBox1->text(i);
		QDir d(t, "*.mp3 *.MP3 *.Mp3 *.mP3");
		QStringList sl = d.entryList();	

		for (QStringList::Iterator j = sl.begin(); j != sl.end(); j++) {
			if (flag) {
				flag = false;
				continue;
			}

			QString u = s + '/' + *j + '=' + t + *j;
			out << u << '\n';
		}
	}
	out.close();

	execlp("mkisofs", "mkisofs", "-v", "-o", ImageLocationEntry->text().latin1(), 
		"-graft-points", "-path-list", tmp, str, 0);
	
	exit(0);
}

void MainWin::DoParentStuff()
{
	readSock = new QSocket();
	readSock->setSocket(readPipe);
	
	connect(readSock, SIGNAL(readyRead()), this, SLOT(ParentHandleRead()));
	progress = new QProgressDialog("Creating CD Image.", "Cancel", 100, this);
	progress->setProgress(0);
	connect(progress, SIGNAL(cancelled()), this, SLOT(SlotCancelled()));
	readingNumbers = false;
}


bool MainWin::CreateImageCheckErrors()
{
	if (!ListBox1->count()) {
		QMessageBox::information(this, "MP3 CD Image Creator", 
			"You must insert at least one directory to the listing.");

		return true;
	}
	
	if (!totalSizeBytes) {
		QMessageBox::information(this, "MP3 CD Image Creator",
			"Your directories must contain MP3 files.");

		return true;
	}

	if (ImageLocationEntry->text().isEmpty()) {
		QMessageBox::information(this, "MP3 CD Image Creator",
			"You must supply a location for your image.");
		
		return true;
	}

/*	if (!QFileInfo(ImageLocationEntry->text()).isWritable()) {
		QMessageBox::information(this, "MP3 CD Image Creator",
			"You must have permission to write to the image location.");
		
		return true;
	}*/
	return false;
}


void MainWin::ParentHandleRead()
{
	while (readSock->canReadLine()) {
		if (!readingNumbers) {
			readSock->readLine();
			readingNumbers = true;
			continue;
		}

		QString s = readSock->readLine();
		s = s.stripWhiteSpace();
		
		s.truncate(s.find(' ') - 4);

		bool isOK;
		int percent = s.toInt(&isOK); 

		if (!isOK) {
			progress->setProgress(100);
			readSock->close();
			wait(0);
		}
		else progress->setProgress(percent);

			
	}
}

void MainWin::SlotCancelled()
{
	readSock->close();
	kill(pid, 15);
	QFile(ImageLocationEntry->text()).remove();
	wait(0);
}

